#!/bin/sh
###############################################################################
# Copyright 2009-2019, Way to the Web Limited
# URL: http://www.configserver.com
# Email: sales@waytotheweb.com
###############################################################################
PATH=$PATH:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin

if [ -e "/usr/local/cpanel/version" ]; then
	echo "Running cmq cPanel installer"
	echo
	sh install.cpanel.sh
elif [ -e "/usr/local/directadmin/directadmin" ]; then
	echo "Running cmq DirectAdmin installer"
	echo
	sh install.directadmin.sh
fi
