.icon-configserver {
	color: #990000;
}
.btn-default:active,
.btn-default:visited,
.btn-default:focus,
.btn-default {
	background:#FFFFFF;
	border-radius:3px;
	border:1px solid #A6C150;
	color:#990000 !important;
}
.btn-default:hover {
	border:1px solid #A6C150;
	background: #F5F5F5;
}
input[type=text], select {
	-webkit-transition: all 0.30s ease-in-out;
	-moz-transition: all 0.30s ease-in-out;
	-ms-transition: all 0.30s ease-in-out;
	-o-transition: all 0.30s ease-in-out;
	transition: all 0.30s ease-in-out;
	border-radius:3px;
	outline: none;
	padding: 3px 0px 3px 3px;
	margin: 5px 1px 3px 0px;
	border: 1px solid #990000;
}
input[type=text]:focus, select:focus {
	box-shadow: 0 0 5px #CC0000;
	padding: 3px 0px 3px 3px;
	margin: 5px 1px 3px 0px;
	border: 1px solid #990000;
}
.td-btn {
	width: 200px;
}
.td-text {
}
th {
	background: #F4F4EA;
}
.table tbody>tr>td {
	vertical-align: middle;
}
.panel-default > .panel-heading-cxs {
	font-weight: bold;
	background: #F4F4EA;
}
.panel-default > .panel-footer-cxs {
	font-weight: bold;
	background: #F4F4EA;
}
#loader {
	position: absolute;
	left: 50%;
	top: 50%;
	z-index: 1;
	margin: -75px 0 0 -75px;
	border: 16px solid #F4F4EA;
	border-radius: 50%;
	border-top: 16px solid #990000;
	border-bottom: 16px solid #990000;
	width: 120px;
	height: 120px;
	-webkit-animation: spin 2s linear infinite;
	animation: spin 2s linear infinite;
}
@-webkit-keyframes spin {
	0% { -webkit-transform: rotate(0deg); }
	100% { -webkit-transform: rotate(360deg); }
}
@keyframes spin {
	0% { transform: rotate(0deg); }
	100% { transform: rotate(360deg); }
}	
